@@ -557,7 +557,7 @@ GRPH2
     EXAMPLES
     * You can modify parameters temporarily as follows.
 
-        before = DCLExt.gl_set_params('lmiss'=>true,'rmiss'=>9999.0)
+        before = DCLExt.gl_set_params({'lmiss'=>true,'rmiss'=>9999.0})
         ....
         DCLExt.gl_set_params(before)     # reset the change
 
@@ -1036,6 +1036,143 @@ module NumRu
       nil
     end
 
+    ############################################################
+    # RELATIVELY INDEPENDENT OF DCL SUBLIBRARIES
+    ############################################################
+
+    # < flow vector package >
+
+    def __truncate(float, order=2)
+      # truncate (round) a floating number with the number digits
+      # specified by "order".
+      # e.g., if order=3, -0.012345 => -0.0123;  6.6666 => 6.67
+      exponent = 10**(-Math::log10(float.abs).floor+order-1)
+      (float * exponent).round.to_f/exponent
+    end
+
+    @@unit_vect_options = Misc::KeywordOptAutoHelp.new(
+      ['vxunit', 0.05, "x unit vect len in V coord. Used only when fxunit is omitted (default)"],
+      ['vyunit', 0.05, "y unit vect len in V coord. Used only when fyunit is omitted (default)"],
+      ['vxuloc', nil, "Starting x position of unit vect"],
+      ['vyuloc', nil, "Starting y position of unit vect"],
+      ['vxuoff', 0.05, "Specify vxuloc by offset from right-bottom corner"],
+      ['vyuoff', 0.0, "Specify vyuloc by offset from right-bottom corner"],
+      ['inplace',true, "Whether to print labels right by the unit vector (true) or below the x axis (false)"],
+      ['rsizet', nil, "Label size(default taken from uz-parameter 'rsizel1')"],
+      ['index',  3," Line index of the unit vector"]
+    )
+
+    def set_unit_vect_options(options)
+      @@unit_vect_options.set(options)
+    end
+
+    @@next_unit_vect_options = nil
+    def next_unit_vect_options(options)
+      if options.is_a?(Hash)
+        @@next_unit_vect_options = options
+      else
+        raise TypeError,"Hash expected"
+      end
+      nil
+    end
+
+    def unit_vect( vxfxratio,   # (V cood length)/(actual length) in x
+		   vyfyratio,   # (V cood length)/(actual length) in y
+		   fxunit=nil,  # If specified, x unit vect len
+		   fyunit=nil,  # If specified, y unit vect len
+		   options=nil )
+      #< options >
+      if @@next_unit_vect_options
+        options = ( options ? @@next_unit_vect_options.update(options) : 
+                              @@next_unit_vect_options )
+        @@next_unit_vect_options = nil
+      end
+      opt = @@unit_vect_options.interpret(options)
+      vxunit = opt['vxunit']
+      vyunit = opt['vyunit']
+      vxuloc = opt['vxuloc']
+      vyuloc = opt['vyuloc']
+      rsizet = opt['rsizet']
+      index = opt['index']
+      
+      #< unit vector >
+      if fxunit
+	vxunit = vxfxratio * fxunit
+      else
+	fxunit = vxunit / vxfxratio
+      end
+      if fyunit
+	vyunit = vyfyratio * fyunit
+      else
+	fyunit = vyunit / vyfyratio
+      end
+      fxunit = __truncate( (uxusv=fxunit) )
+      fyunit = __truncate( (uyusv=fyunit) )
+      vxunit = vxunit * (fxunit/uxusv)
+      vyunit = vyunit * (fyunit/uyusv)
+      if !(vxuloc && vyuloc)
+	vx0,vx1,vy0,vy1 = DCL.sgqvpt
+	vxuloc = vx1 + opt['vxuoff'] if !vxuloc
+	vyuloc = vy0 + opt['vyuoff'] if !vyuloc
+      end
+      DCL.sglazv( vxuloc, vyuloc,  vxuloc+vxunit, vyuloc,        1, index )
+      DCL.sglazv( vxuloc, vyuloc,  vxuloc,        vyuloc+vyunit, 1, index )
+
+      #< labelling >
+      sfxunit = sprintf("%.2g",fxunit)
+      sfyunit = sprintf("%.2g",fyunit)
+      rsizet = DCL.uzpget('rsizel1') if !rsizet 
+      if opt['inplace']
+	DCL.sgtxzv(vxuloc, vyuloc-1.2*rsizet,
+		   sfxunit, rsizet, 0, -1, index)
+	DCL.sgtxzv(vxuloc+1.2*rsizet, vyuloc+0.5*rsizet,
+		   sfyunit, rsizet, 90, -1, index)
+      else
+	msg= "UNIT VECTOR X:#{sfxunit} Y:#{sfyunit}"
+	before = uz_set_params({'rsizec1'=>rsizet})
+	DCL.uxsttl('b',' ',0.0)
+	DCL.uxsttl('b',msg,0.0)
+	uz_set_params(before)
+      end
+    end
+
+    def flow_vect( fx, fy, factor=1.0, xintv=1, yintv=1)
+      raise ArgumentError,"Expect 2D arrays" if fx.rank != 2 || fy.rank != 2
+      raise ArgumentError,"fx.shape != fy.shape" if fx.shape != fy.shape
+      raise ArgumentError,"xintv must be a positive integer" if xintv < 0
+      raise ArgumentError,"yintv must be a positive integer" if yintv < 0
+      nx, ny = fx.shape
+      if xintv >= 2
+	idx = NArray.int(nx/xintv).indgen!*xintv  # [0,xintv,2*xintv,..]
+	fx = fx[idx, true]
+	fy = fy[idx, true]
+      end
+      if yintv >= 2
+	idx = NArray.int(ny/yintv).indgen!*yintv  # [0,yintv,2*yintv,..]
+	fx = fx[true, idx]
+	fy = fy[true, idx]
+      end
+      nx, ny = fx.shape  # again, because of xintv & yintv 
+      vx0,vx1,vy0,vy1 = DCL.sgqvpt
+      ux0,ux1,uy0,uy1 = DCL.sgqwnd
+      dvx = (vx1-vx0)/nx
+      dvy = (vy1-vy0)/ny
+      ax = (vx1-vx0)/(ux1-ux0)   # factor to convert from U to V coordinate
+      ay = (vy1-vy0)/(uy1-uy0)   # factor to convert from U to V coordinate
+      fxmx = fx.abs.max
+      fymx = fy.abs.max
+      raise "fx has no data or all zero" if fxmx == 0
+      raise "fy has no data or all zero" if fymx == 0
+      cn = [ dvx/(ax*fxmx),  dvy/(ay*fymx) ].min  # normarization constant
+      vxfxratio = factor*cn*ax
+      vyfyratio = factor*cn*ay
+      before = ug_set_params( {'LNRMAL'=>false, 'LMSG'=>false,
+			       'XFACT1'=>1.0, 'YFACT1'=>1.0} )
+      DCL.ugvect( vxfxratio*fx, vyfyratio*fy )
+      ug_set_params( before )
+      unit_vect_info = [ vxfxratio, vyfyratio, fxmx, fymx ]
+      return unit_vect_info
+    end
   end
 
 
@@ -1440,6 +1577,8 @@ module NumRu
           ['title', nil, 'Title of the figure(if nil, internally determined)'],
           ['annotate', true, 'if false, do not put texts on the right margin even when newframe==true'],
           ['ltone', true, 'Same as udpack parameter ltone'],
+          ['tonf', true, 'Use DCL.uetonf instead of DCL.uetone'],
+          ['tonc', true, 'Use DCL.uetonc instead of DCL.uetone'],
           ['clr_min', nil,  'if an integer (in 10..99) is specified, used as the mimimum color number. (the same can be done by setting the uepack parameter "icolor1")'],
           ['clr_max', nil,  'if an integer (in 10..99) is specified, used as the maximum color number. (the same can be done by setting the uepack parameter "icolor2")'],
           ['transpose', false, 'if true, exchange x and y axes'],
@@ -1470,15 +1609,33 @@ module NumRu
         backup = @@set_tone_levels
         _set_tone_levels_(opts)
         @@set_tone_levels = backup
-        DCL.uetone(gp.data.val)
+	if opts['tonf']
+	  DCL.uetonf(gp.data.val)
+	elsif opts['tonc']
+	  DCL.uetonc(gp.data.val)
+	else
+	  DCL.uetone(gp.data.val)
+	end
       elsif opts['patterns']
 	raise "option 'patterns' is not effective unless 'levels' is specified"
       elsif !@@set_tone_levels && opts['ltone']
         DCLExt.ue_set_linear_levs(gp.data.val, opts)
-        DCL.uetone(gp.data.val)
+	if opts['tonf']
+	  DCL.uetonf(gp.data.val)
+	elsif opts['tonc']
+	  DCL.uetonc(gp.data.val)
+	else
+	  DCL.uetone(gp.data.val)
+	end
         DCL.ueitlv
       else
-        DCL.uetone(gp.data.val)
+	if opts['tonf']
+	  DCL.uetonf(gp.data.val)
+	elsif opts['tonc']
+	  DCL.uetonc(gp.data.val)
+	else
+	  DCL.uetone(gp.data.val)
+	end
       end
       if newframe
         axes(xax, yax)  # axes should be drawn after uetone
@@ -1490,6 +1647,86 @@ module NumRu
       nil
     end
 
+    # < vector >
+
+    def set_unit_vect_options(options)
+      DCLExt.set_unit_vect_options(options)
+    end
+    def next_unit_vect_options(options)
+      DCLExt.next_unit_vect_options(options)
+    end
+
+    def vector(fx, fy, newframe=true, options=nil)
+      if newframe!=true && newframe!=false
+        raise ArgumentError, "2nd arg (newframe) must be true or false"
+      end
+      if ! defined?(@@vector_options)
+        @@vector_options = Misc::KeywordOptAutoHelp.new(
+          ['title', nil, 'Title of the figure(if nil, internally determined)'],
+          ['annotate', true, 'if false, do not put texts on the right margin even when newframe==true'],
+          ['transpose', false, 'if true, exchange x and y axes'],
+          ['flow_vect', true, 'If true, use DCLExt::flow_vect to draw vectors; otherwise, DCL::ugvect is used.'],
+          ['xintv', 1, '(Effective only if flow_vect) interval of data sampling in x'],
+          ['yintv', 1, '(Effective only if flow_vect) interval of data sampling in y'],
+          ['factor', 1.0, '(Effective only if flow_vect) scaling factor to strech/reduce the arrow lengths'],
+          ['unit_vect', false, 'Show the unit vector'],
+          ['max_unit_vect', false, '(Effective only if flow_vect && unit_vect) If true, use the maximum arrows to scale the unit vector; otherwise, normalize in V coordinate.']
+        )
+      end
+      opts = @@vector_options.interpret(options)
+      fx = fx.first2D.copy
+      fy = fy.first2D.copy
+      sh = fx.shape
+      if sh != fy.shape
+	raise ArgumentError, "shapes of fx and fy do not agree with each other"
+      end
+      fx = fx.transpose(1,0) if opts['transpose']
+      fy = fy.transpose(1,0) if opts['transpose']
+      if ((xi=opts['xintv']) >= 2)
+	idx = NArray.int(sh[0]/xi).indgen!*xi     # [0,xi,2*xi,..]
+	fx = fx[idx, true]
+	fy = fy[idx, true]
+      end
+      if ((yi=opts['xintv']) >= 2)
+	idx = NArray.int(sh[1]/yi).indgen!*yi     # [0,yi,2*yi,..]
+	fx = fx[true, idx]
+	fy = fy[true, idx]
+      end
+      xax = fx.coord(0)
+      yax = fy.coord(1)
+      if newframe
+        fig(xax, yax)
+        axes(xax, yax)
+	if opts['title']
+	  ttl = opts['title']
+	else
+	  fxnm = fx.data.get_att('long_name') || fx.name
+	  fynm = fy.data.get_att('long_name') || fy.name
+	  ttl =   '('+fxnm+','+fynm+')'
+	end
+        title( ttl )
+        annotate(fx.lost_axes) if opts['annotate']
+      end
+      DCL.uwsgxa(xax.val)
+      DCL.uwsgya(yax.val)
+      if opts['flow_vect']
+	uninfo = DCLExt.flow_vect(fx.val, fy.val, opts['factor'] )
+	if opts['unit_vect']
+	  if opts['max_unit_vect']
+	    DCLExt.unit_vect(*uninfo)
+	  else
+	    DCLExt.unit_vect(*uninfo[0..1])
+	  end
+	end
+      else
+	before=DCLExt.ug_set_params({'lunit'=>true}) if opts['unit_vect']
+	DCL.ugvect(fx.val, fy.val)
+	DCLExt.ug_set_params(before) if opts['unit_vect']
+      end
+      nil
+    end
+
+
   end
 
 end
@@ -1547,7 +1784,7 @@ if $0 == __FILE__
   DCL.sgpset('lcntl', false)
   DCL.sgpset('lfull',true)
   DCL.sgpset('lfprop',true)
-  DCL.uzfact(0.7)
+  DCL.uzfact(0.9)
   #/ graph 1 /
   GGraph.set_fig('viewport'=>[0.25,0.75,0.12,0.62])
   GGraph.line(gphys.cut(true,35,true).average(0), true)
@@ -1603,6 +1840,17 @@ if $0 == __FILE__
   GGraph.set_fig('viewport'=>[0.3,0.7,0.1,0.6])
   GGraph.contour(gphys,true, 'min'=>0, 'coloring'=>true, 'transpose'=>true)
   GGraph.tone(gphys, false, 'levels'=>[10,20], 'patterns'=>[80999], 'transpose'=>true)
+  #/ graph 19 /
+  GGraph.set_fig('viewport'=>[0.25,0.75,0.15,0.6])
+  GGraph.vector(gphys+10,gphys+10,true,
+	{'unit_vect'=>true, 'xintv'=>2, 'yintv'=>2})
+  #/ graph 18 /
+  GGraph.next_unit_vect_options({'inplace'=>false,'vxuloc'=>0.17,'vyuloc'=>0.07})
+  GGraph.vector(gphys+10,gphys-10,true,
+	{'unit_vect'=>true, 'max_unit_vect'=>true, 'xintv'=>2, 'yintv'=>2})
+  #/ graph 19 /
+  GGraph.vector(gphys+10,gphys+10,true,
+	{'unit_vect'=>true, 'flow_vect'=>false, 'xintv'=>2, 'yintv'=>2})
 
   DCL.grcls
 
