@@ -185,6 +185,24 @@ module NumRu
 	 @aux ? @aux.keys : []
       end
 
+      def to_gphys(datavary1d=nil)
+	# To form a 1-dimensional GPhys
+	# (Dependent on GPhys&Grid, unlike other methods Axis,
+        # so the test program is in gphys.rb)
+	# Arguments
+	#   * datavary1d (nil or VArray): 1D VArray with the same
+        #     size as the axis. If omitted, @pos (coordinate variable
+        #     of the variable) is used.
+	if !datavary1d
+	  datavary1d = @pos.copy
+	else
+	  if datavary1d.rank != 1 || datavary1d.length != length
+	      raise ArgumentError, "Must be 1D and same size"
+	  end
+	end
+	GPhys.new( Grid.new(self), datavary1d )
+      end
+
       def [] (slicer)
 	 if ! @pos
 	    raise "pos has not been set. Forgot set_pos_to_(center|bounds)?"
