@@ -79,6 +79,21 @@ module NumRu
       def replace_val(v)
 	 @data.replace_val(v)
       end
+
+      def att_names
+	@data.att_names
+      end
+      def get_att(name)
+	@data.get_att(name)
+      end
+      def set_att(name, val)
+	@data.set_att(name, val)
+      end
+      def del_att(name)
+	@data.del_att(name)
+      end
+      alias put_att set_att
+
       def units
 	@data.units
       end
@@ -86,6 +101,15 @@ module NumRu
 	@data.units= units
       end
 
+      def convert_units(to)
+	# ==NOTE: 
+	#    * VArray#convert_units does not copy data if to == @data.units
+	#    * @grid is shared with self (no duplication)
+        #    Thus, use GPhys#copy to separate all sub-objects (deep clone).
+	data = @data.convert_units(to)  
+	GPhys.new(@grid, data)
+      end
+
       def [](*slicer)
 	 if slicer.length==1 && slicer[0].is_a?(Hash) && 
 	    slicer[0].keys[0].is_a?(String)
@@ -473,4 +497,10 @@ if $0 == __FILE__
    ax = gpz3.axis(1)
    p ax.to_gphys
    p ax.to_gphys( ax.cell_bounds[0..-2].copy )
+
+   print "convert units\n"
+   gpz3.units = 'm'
+   p gpz3.units
+   gpz3k = gpz3.convert_units('km')
+   p gpz3k.units, gpz3.val, gpz3k.val
 end
