@@ -73,10 +73,21 @@ module NumRu
 
       def delete_axes( at, deleted_by=nil )
 	case at
+        when String
+          at = [dim_index(at)]
 	when Numeric
 	  at = [at]
 	when Array
-	  at = at.dup
+          at = at.collect{|x|
+            case x
+            when String
+              dim_index(x)
+            when Integer
+              x
+            else
+              raise ArgumentError,"'at' must consist of Integer and/or String"
+            end
+          }
 	else
 	  raise TypeError, "1st arg not an Array"
 	end
@@ -269,6 +280,18 @@ module NumRu
 	 self
       end
 
+      def transpose( *dims )
+	if dims.sort != NArray.int(rank).indgen!.to_a
+	  raise ArgumentError, 
+            "Args must a permutation of 0..rank-1 (eg, if 3D 2,1,0; 1,0,2;etc)"
+	end
+	axes = Array.new
+	for i in 0...rank
+	  axes[i] = @axes[dims[i]]
+	end
+	Grid.new(*axes)
+      end
+
       # Define operations along each axis --- The following defines
       # instance methods such as "average" and "integrate":
 
@@ -349,6 +372,7 @@ if $0 == __FILE__
    p grid.delete_axes([0,1]).lost_axes
    p grid.delete_axes([0,1], 'mean').lost_axes
 
+   p grid, grid.transpose(1,0)
 
 end
 
