@@ -173,7 +173,7 @@ module NumRu
 	attr.delete('FORTRAN_format')
 	attr.delete('add_offset')
 	attr.delete('scale_factor')
-	attr.delete('long_name')
+	##attr.delete('long_name')
       end
       private :_op_attr_edit
 
