@@ -273,7 +273,7 @@ These methods returns a NArray (not a VArray).
 	    @attr = attr
 	 when VArray
 	    vary = attr
-	    @attr = vary.attr.copy
+	    @attr = vary.attr_copy
 	 when Hash
 	    @attr = NumRu::Attribute.new
 	    attr.each{|key,val| @attr[key]=val}
@@ -328,7 +328,7 @@ These methods returns a NArray (not a VArray).
 	  self
 	else
 	  # subclass --> VArray
-	  VArray.new(narray, attr.copy, name)
+	  VArray.new(narray, attr_copy, name)
 	end
       end
 
@@ -377,7 +377,7 @@ These methods returns a NArray (not a VArray).
       ### < basic parts invariant in subclasses > ###
 
       def copy(to=nil)
-	 attr = self.attr.copy( (to ? to.attr : to) )
+	 attr = self.attr_copy( (to ? to.attr : to) )
 	 val = self.val
 	 if self.class==VArray && !self.mapped? && (to==nil || to.class==VArray)
 	    val = val.dup
@@ -431,6 +431,10 @@ These methods returns a NArray (not a VArray).
       end
       protected :attr
 
+      def attr_copy(to=nil)
+	attr.copy(to)
+      end
+
       def att_names
 	attr.keys
       end
@@ -459,6 +463,20 @@ These methods returns a NArray (not a VArray).
 	units
       end
 
+      def convert_units(to)
+	if ! to.is_a?(Units)
+	  to = Units.new(to)
+	end
+	myunits = self.units
+	if myunits != to
+	  gp = myunits.convert2(self, to)
+	  gp.units = to
+	  gp
+	else
+	  self   # returns self (no duplication)
+	end
+      end
+
       def [](*slicer)
 	 mapping = SubsetMapping.new(self.shape_current, slicer)
 	 VArray.new.initialize_mapping(mapping, self)
@@ -490,7 +508,7 @@ These methods returns a NArray (not a VArray).
       end
 
       def transpose(*dims)
-	VArray.new( val.transpose(*dims), @attr.copy, name )
+	VArray.new( val.transpose(*dims), attr_copy, name )
       end
 
       ### < NArray methods > ###
@@ -499,14 +517,14 @@ These methods returns a NArray (not a VArray).
       ## ToDo: coerce
 
       def coerce(other)
-	oattr = self.attr.copy
+	oattr = self.attr_copy
 	case other
 	when UNumeric
 	  oattr['units'] = other.units.to_s
 	  na_other, = NArray.new(self.typecode, 1).coerce(other.val)   # scalar
 	  c_other = VArray.new(na_other, oattr, self.name)
 	else
-	  oattr['units'] = '1'    # Assume the other to be non-dimensional
+	  oattr['units'] = self.get_att('units')     # Assume the same units
 	  case other
 	  when Numeric
 	    na_other, = NArray.new(self.typecode, 1).coerce(other)   # scalar
@@ -566,7 +584,7 @@ These methods returns a NArray (not a VArray).
       for f in Math_funcs_nondim
 	 eval <<-EOS, nil, __FILE__, __LINE__+1
 	 def #{f}(*arg)
-            newattr = self.attr.copy
+            newattr = self.attr_copy
             newattr['units'] = '1'
 	    case arg.length
 	    when 0
@@ -583,7 +601,7 @@ These methods returns a NArray (not a VArray).
       for f in Math_funcs_radian
 	 eval <<-EOS, nil, __FILE__, __LINE__+1
 	 def #{f}(*arg)
-            newattr = self.attr.copy
+            newattr = self.attr_copy
             newattr['units'] = 'rad'
 	    case arg.length
 	    when 0
@@ -599,7 +617,7 @@ These methods returns a NArray (not a VArray).
          EOS
       end
       def sqrt
-	va = VArray.new( Misc::EMath.sqrt(self.val), self.attr.copy, self.name )
+	va = VArray.new( Misc::EMath.sqrt(self.val), self.attr_copy, self.name )
 	va.units = units**Rational(1,2)
 	va
       end
@@ -610,12 +628,12 @@ These methods returns a NArray (not a VArray).
 	    case other
             when VArray, UNumeric
                ary = self.val#{f}(other.val)
-	       va = VArray.new( ary, self.attr.copy, self.name )
+	       va = VArray.new( ary, self.attr_copy, self.name )
 	       va.units= self.units#{f}(other.units)
                va
 	    when Numeric, NArray, NArrayMiss, Array
 	       ary = self.val#{f}(other)
-	       va = VArray.new( ary, self.attr.copy, self.name )
+	       va = VArray.new( ary, self.attr_copy, self.name )
 	       if "#{f}" == "**" 
 		 va.units= self.units#{f}(other)
                end
@@ -633,10 +651,10 @@ These methods returns a NArray (not a VArray).
 	    case other
             when VArray, UNumeric
                ary = self.val#{f}( other.units.convert2(other.val,units) )
-	       VArray.new( ary, self.attr.copy, self.name )
+	       VArray.new( ary, self.attr_copy, self.name )
 	    when Numeric, NArray, NArrayMiss, Array
 	       ary = self.val#{f}(other)
-	       VArray.new( ary, self.attr.copy, self.name )
+	       VArray.new( ary, self.attr_copy, self.name )
             else
                c_me, c_other = other.coerce(self)
                c_me#{f}(c_other)
@@ -650,10 +668,10 @@ These methods returns a NArray (not a VArray).
 	    case other
             when VArray, UNumeric
                ary = self.val#{f}(other.val)
-	       VArray.new( ary, self.attr.copy, self.name )
+	       VArray.new( ary, self.attr_copy, self.name )
 	    when Numeric, NArray, NArrayMiss, Array
 	       ary = self.val#{f}(other)
-	       VArray.new( ary, self.attr.copy, self.name )
+	       VArray.new( ary, self.attr_copy, self.name )
             else
                c_me, c_other = other.coerce(self)
                c_me#{f}(c_other)
@@ -697,7 +715,7 @@ These methods returns a NArray (not a VArray).
 	 eval <<-EOS, nil, __FILE__, __LINE__+1
 	 def #{f}
             ary = #{f.delete("@")} self.val
-	    VArray.new( ary, self.attr.copy, self.name )
+	    VArray.new( ary, self.attr_copy, self.name )
 	 end
 	 EOS
       end
@@ -707,7 +725,7 @@ These methods returns a NArray (not a VArray).
       for f in NArray_type1_methods
 	 eval <<-EOS, nil, __FILE__, __LINE__+1
 	 def #{f}(*args)
-            newattr = self.attr.copy
+            newattr = self.attr_copy
             newattr['units'] = '1' if "#{f}"=="angle" || "#{f}"=="arg"
 	    VArray.new(self.val.#{f}(*args), newattr, self.name )
 	 end
@@ -747,7 +765,7 @@ These methods returns a NArray (not a VArray).
 	 def #{f}(*args)
             result = self.val.#{f}(*args)
             if result.is_a?(NArray) || result.is_a?(NArrayMiss)
-	       VArray.new(result , self.attr.copy, self.name )
+	       VArray.new(result , self.attr_copy, self.name )
             else
 	       UNumeric[result, units]    # used to be 'result' (not UNumeric)
             end
@@ -840,4 +858,8 @@ if $0 == __FILE__
    p '##', vkg.val, vmul.get_att('units'), vmul.units
    p '##', (vkg + vg).val, (vg + vkg).val, vkg > vg
 
+   p '*convert_units*'
+   p vg.units,vg.val
+   vkg = vg.convert_units('kg')
+   p vkg.units,vkg.val
 end
